@@ -26,7 +26,7 @@
  * 
  * derived from ax25-utils/listen.c
  *
- * portions Copyright (c) 1996,1997,1999 Alan Crosswell
+ * portions Copyright (c) 1996,1997,1999,2001 Alan Crosswell
  * Alan Crosswell, N2YGK
  * 144 Washburn Road
  * Briarcliff Manor, NY 10510, USA
@@ -59,15 +59,17 @@
 #include <linux/rose.h>
 #include <signal.h>
 #include <errno.h>
-#include "axconfig.h"
-#include "axutils.h"
+#include <netax25/axconfig.h>
+#include <netax25/axlib.h>
+
 #include "mic_e.h"
+#include "libax25ext.h"
 
 #ifndef PACKAGE
 #define PACKAGE "aprsdigi"
 #endif
 #ifndef VERSION
-#define VERSION "$Revision: 1.28 $";
+#define VERSION "$Revision: 1.31 $";
 #endif
 
 /* some defines that really belong in a header file! */
@@ -156,7 +158,7 @@ static char Dirs[5] = "NSEW";
 static char *Tag = NULL;	/* tag onto end of posit in SSID mode */
 static int Taglen = 0;
 static int Idinterval = (9*60)+30; /* default to 9:30 */
-static int Keep = 20;		/* seconds to remember for dupe test */
+static int Keep = 28;		/* seconds to remember for dupe test */
 static int I_flags = 0;		/* interface default flags */
 #define I_NEED_ID 1		/* need to ID */
 #define SUBST_MYCALL 2		/* replace digi alias w/mycall */
@@ -304,6 +306,7 @@ rx_loop()
 static int rx_nodigi(struct stuff *s,int r);
 static int rx_dupe(struct stuff *s);
 static int rx_to_me(struct stuff *s);
+static int rx_from_me(struct stuff *s);
 static int rx_mice(struct stuff *s);
 static int rx_flood(struct stuff *s);
 static int rx_digi(struct stuff *s);
@@ -323,15 +326,20 @@ rx_packet(struct interface *i,	/* which interface received on */
   s.len = len;
 
   /* parse the raw kiss frame */
-  r = parse_kiss(&s.cp,&s.len,&s.in);
+  r = parse_raw_ax25(&s.cp,&s.len,&s.in);
   if (Verbose) {
-    fprintf(stderr,"%s: RX: ",s.i->port);
+    char buf[20];
+    time_t tick = time(0);
+    strftime(buf,sizeof(buf),"%T",gmtime(&tick));
+    fprintf(stderr,"%s %s: RX: ",buf,s.i->port);
     print_it(stderr,&s.in,s.cp,s.len);
   }
 
   /* go through a bunch of choices until we eat the packet or die:-) */
 
-  if (rx_mice(&s))		/* mic-E special handling */
+  if (rx_from_me(&s))		/* don't digi my own repeated beacons! */
+    return;
+  else if (rx_mice(&s))		/* mic-E special handling */
     return;
   else if (rx_nodigi(&s,r))	/* Nothing to digi? */
     return;
@@ -417,7 +425,7 @@ calltab_entry(ax25_address *callsign,int *flags)
   for (c = calltab; c; c = c->next) {
     if ((c->flags&C_IS_FLOOD 
 	 && (*flags = floodcmp(c->callsign,c->floodlen,callsign)))
-	|| (ax25cmp(c->callsign,callsign) == 0)) {
+	|| (ax25_cmp(c->callsign,callsign) == 0)) {
       *flags |= c->flags;	/* add stuff floodcmp doesn't know about */
       return c;
     }
@@ -487,6 +495,7 @@ rx_to_me(struct stuff *s)
 
   if (l) {
     result = 1;
+    ++s->i->stats.rx_ign;
   }
   if (Verbose) {
     fprintf(stderr,"packet is%saddressed to me.\n",result?" ":" not ");
@@ -494,6 +503,25 @@ rx_to_me(struct stuff *s)
   return result;
 }  
 
+static int
+rx_from_me(struct stuff *s)
+{
+  int n,result = 0;
+  struct interface *iface;
+
+  for (n = 0, iface = Intf; n < N_intf; n++, iface++) {
+    if (ax25_cmp(&MYCALL(n),&s->in.ax_from_call) == 0) {
+      result=1;
+      ++s->i->stats.rx_ign;
+    }
+  }
+
+  if (Verbose) {
+    fprintf(stderr,"packet is%sfrom me.\n",result?" ":" not ");
+  }
+  return result;
+}  
+
 /* 
  * MIC-E SSID path selection only applies if:
  *  1. 1st byte of info field must be one of the "Mic-E cookies."
@@ -532,7 +560,7 @@ rx_mice(struct stuff *s)
 	s->out.ax_digi_call[0].ax25_call[ALEN] |= SSID&(ssid<<1);
 	if (Verbose) {
 	  fprintf(stderr,"Flooding: setting path to %s\n",
-		  ax2asc(&s->out.ax_digi_call[0]));
+		  ax25_ntoa_pretty(&s->out.ax_digi_call[0]));
 	}
       } else {		/* not in a flooding network */
 	int startat,i;
@@ -591,7 +619,7 @@ rx_flood(struct stuff *s)
 
     if (Verbose) {
       fprintf(stderr,"Got a flooding %s route.\n",
-	      ax2asc(&s->in.ax_digi_call[s->in.ax_next_digi]));
+	      ax25_ntoa_pretty(&s->in.ax_digi_call[s->in.ax_next_digi]));
     }
 
     /* flooding algorithm always kills dupes.  If kill_dupes
@@ -626,7 +654,7 @@ rx_flood(struct stuff *s)
     }
     if (Verbose) {
       fprintf(stderr,"Rewriting it as %s:\n",
-	      ax2asc(&s->out.ax_digi_call[s->out.ax_next_digi]));
+	      ax25_ntoa_pretty(&s->out.ax_digi_call[s->out.ax_next_digi]));
     }
     ++s->i->stats.flood;
     if (xmit(s) < 0) 
@@ -696,7 +724,7 @@ char *str;
 
   bzero(calls, sizeof(*calls));
   sprintf(buf,"dummy via %s",str);
-  return convert_call(buf,calls);
+  return ax25_aton(buf,calls);
 }
 
 static int
@@ -711,7 +739,7 @@ char *str;
   bzero(calls,ncalls*sizeof(*calls));
   cp = strtok(str," \t\n,");
   for (i = 0; cp && i < ncalls; i++) {
-    if (convert_call_entry(cp,calls[i].ax25_call) < 0)
+    if (ax25_aton_entry(cp,calls[i].ax25_call) < 0)
       return -1;
     cp = strtok(NULL," \t\n,");
   }
@@ -729,11 +757,11 @@ print_it(FILE *f,
 
   if (f == NULL)
     return;
-  strncpy(asc_to,ax2asc(&calls->ax_to_call),sizeof(asc_to));
-  strncpy(asc_from,ax2asc(&calls->ax_from_call),sizeof(asc_from));
+  strncpy(asc_to,ax25_ntoa_pretty(&calls->ax_to_call),sizeof(asc_to));
+  strncpy(asc_from,ax25_ntoa_pretty(&calls->ax_from_call),sizeof(asc_from));
   fprintf(f,"%s>%s",asc_from,asc_to);
   for (j = 0; j < calls->ax_n_digis; j++) {
-    fprintf(f,",%s%s",ax2asc(&calls->ax_digi_call[j]),
+    fprintf(f,",%s%s",ax25_ntoa_pretty(&calls->ax_digi_call[j]),
 	    (calls->ax_digi_call[j].ax25_call[ALEN]&REPEATED
 		&& (j == calls->ax_next_digi-1))?"*":"");
   }
@@ -767,12 +795,12 @@ reformat(struct stuff *s,	/* "class data":-) */
   vecp[1] = mic2;
   /* does xmit intf want mic-E translation? */
   if (i->i_flags&MICE_XLATE && 
-      fmt_mic_e(ax2asc(&s->out.ax_to_call),s->cp,s->len,
+      fmt_mic_e(ax25_ntoa_pretty(&s->out.ax_to_call),s->cp,s->len,
 		mic1,&vecl[0],mic2,&vecl[1],now)) {
     s->out.ax_to_call = Aprscall; /* replace compressed lat w/"APRS" */
     return (vecl[0]>0)+(vecl[1]>0);
   } else if (i->i_flags&X1J4_XLATE &&   /* wantx1j4 translation? */
-	     fmt_x1j4(ax2asc(&s->out.ax_to_call),s->cp,s->len,
+	     fmt_x1j4(ax25_ntoa_pretty(&s->out.ax_to_call),s->cp,s->len,
 		      mic1,&vecl[0],mic2,&vecl[1],now)) {
     return (vecl[0]>0)+(vecl[1]>0);
   } else {			/* no reformat; just pass cp, len thru */
@@ -798,7 +826,7 @@ xmit(struct stuff *s)
   /* transmit the packet on the interface(s) of the next callsign */
   if ((c = calltab_entry(&s->out.ax_digi_call[s->out.ax_next_digi],&thisflags)) == NULL) {
     fprintf(stderr,"xmit: Assertion failed: call %s not found\n",
-	    ax2asc(&s->out.ax_digi_call[s->out.ax_next_digi]));
+	    ax25_ntoa_pretty(&s->out.ax_digi_call[s->out.ax_next_digi]));
     return -1;
   }
   for(l = c->l; l; l = l->next) { /* loop over each tx interface */
@@ -837,7 +865,7 @@ xmit(struct stuff *s)
 	}
 	++calls.ax_next_digi;
       }
-      gen_kiss(&op,&oleft,&calls);	/* generate the kiss header */
+      gen_raw_ax25(&op,&oleft,&calls);	/* generate the kiss header */
       add_text(&op,&oleft,vecp[n],vecl[n],l->i->tag,l->i->taglen); /* fill in the info field */	
 
       olen = sizeof(obuf) - oleft;
@@ -927,7 +955,7 @@ dupe_packet(struct ax_calls *calls,u_char *cp,int len)
      * heard again or let it age out so that it can get digi'd
      * periodically?  For example, if a station beacons once every 15
      * seconds, then only the first packet received ever will be
-     * digi'd if the "memory" is 20 sec.  Good example of where this
+     * digi'd if the "memory" is 28 sec.  Good example of where this
      * could be a problem is a MIC-E beaconing an emergency.  By not
      * refreshing the timestamp, the desired behavior of reducing channel
      * clutter is still achieved.
@@ -980,19 +1008,33 @@ print_dupes(void) {
 /*
  * is packet looping?
  * e.g. FOO,mycall,WIDE*,mycall,somecall....
- * This will break some people's traces:-)  So will kill_dupes.
+ * Only look for one of my already repeated real callsigns (e.g. MYCALL(n)).
+ *
+ * Search through the list of already-repeated digipeaters and compare
+ * each to one of my interface callsigns (since these are the ones that
+ * replace generic callsigns).
  */
 
 static int
 loop_packet(struct ax_calls *calls,u_char *cp,int len)
 {
   int n, flags;
+  int j;
+  struct interface *iface;
   struct callsign_list *cl;
 
   for (n = 0; n < calls->ax_next_digi; n++) {
-    if ((cl = calltab_entry(&calls->ax_digi_call[n],&flags))
-	&& flags == C_IS_NOFLOOD)
-      return 1;
+    if (calls->ax_digi_call[n].ax25_call[ALEN]&REPEATED) {
+      for (j = 0, iface = Intf; j < N_intf; j++, iface++) {
+	if (ax25_cmp(&MYCALL(j),&calls->ax_digi_call[n]) == 0) {
+	  if (Verbose)
+	    fprintf(stderr,"loop_packet: found my call %s at digi_call[%d]\n",
+		    ax25_ntoa_pretty(&calls->ax_digi_call[n]),n);
+	  return 1;
+	}
+      }
+    } else
+      return 0;			/* last repeated call */
   }
   return 0;
 }
@@ -1067,21 +1109,21 @@ set_id()
     int oleft = sizeof(iface->idbuf);
 
     bzero(&id,sizeof(id));
-    convert_call_entry("ID",id.ax_to_call.ax25_call);
+    ax25_aton_entry("ID",id.ax_to_call.ax25_call);
     id.ax_to_call.ax25_call[ALEN] |= C; /* it's a command */
     id.ax_from_call = MYCALL(n);
     id.ax_type = UI;
     id.ax_pid = PID_NO_L3;
     id.ax_n_digis = 0;
 
-    gen_kiss(&op,&oleft,&id);	/* generate the kiss header */
+    gen_raw_ax25(&op,&oleft,&id);	/* generate the kiss header */
     *idinfo = '\0';
     for (i = 0; i < iface->n_aliases; i++) {
-      strcat(idinfo,ax2asc(&iface->aliases[i]));
+      strcat(idinfo,ax25_ntoa_pretty(&iface->aliases[i]));
       strcat(idinfo,"/R ");
     }
     for (i = 0; i < N_floods; i++) {
-      strcat(idinfo,ax2asc(&Floods[i].call));
+      strcat(idinfo,ax25_ntoa_pretty(&Floods[i].call));
       strcat(idinfo,"n-n/R ");
     }
     add_text(&op,&oleft,idinfo,strlen(idinfo),0,0);
@@ -1213,7 +1255,7 @@ do_opts(int argc, char **argv)
 	fprintf(stderr,"too many flooding aliases!\n");
 	exit(1);
       }
-      if (convert_call_entry(optarg,Floods[N_floods].call.ax25_call) < 0) {
+      if (ax25_aton_entry(optarg,Floods[N_floods].call.ax25_call) < 0) {
 	fprintf(stderr,"Don't understand callsign %s\n",optarg);
 	exit(1);
       }
@@ -1256,7 +1298,7 @@ do_opts(int argc, char **argv)
       break;
     case 'k':
       if ((Keep = atoi(optarg)) <= 0)
-	Keep = 20;		/* default keep is 20 */
+	Keep = 28;		/* default keep is 28 */
       break;
     case 'l':
       Logfile = optarg;		/* log digipeated packets */
@@ -1347,9 +1389,9 @@ do_ports()
   int n;
   struct interface *i;
 
-  convert_call_entry("APRS",Aprscall.ax25_call);
-  convert_call_entry("WIDE",Widecall.ax25_call);
-  convert_call_entry("trace",Trace_dummy.ax25_call);
+  ax25_aton_entry("APRS",Aprscall.ax25_call);
+  ax25_aton_entry("WIDE",Widecall.ax25_call);
+  ax25_aton_entry("trace",Trace_dummy.ax25_call);
   if (ax25_config_load_ports() == 0)
     fprintf(stderr, "aprsdigi: no AX.25 port data configured\n");
   
@@ -1458,7 +1500,7 @@ check_config()
   int n, j;
   struct callsign_list *cl;
   
-  printf("Linux APRS(tm) digipeater\nportions Copyright (c) 1996,1997,1999 Alan Crosswell, n2ygk@weca.org\n");
+  printf("Linux APRS(tm) digipeater\nportions Copyright (c) 1996,1997,1999,2001 Alan Crosswell, n2ygk@weca.org\n");
   printf("Version: aprsdigi %s-%s\n",PACKAGE,VERSION);
   printf("This is free software covered under the GNU Public License.\n");
   printf("There is no warranty.  See the file COPYING for details.\n\n");
@@ -1467,7 +1509,7 @@ check_config()
   for (i = Intf, n = 0; n < N_intf; i++,n++) {
     printf("%-9.9s",i->dev);
     for (j = 0; j < i->n_aliases; j++) {
-      printf(" %-9.9s",ax2asc(&i->aliases[j]));
+      printf(" %-9.9s",ax25_ntoa_pretty(&i->aliases[j]));
     }
     printf("\n");
   }
@@ -1488,7 +1530,7 @@ check_config()
     struct interface_list *l;
     char t[10];
 
-    strcpy(t,ax2asc(cl->callsign));
+    strcpy(t,ax25_ntoa_pretty(cl->callsign));
     if (cl->flags&C_IS_FLOOD)
       strcat(t,"n-n");		/* indicate a WIDEn-n */
     printf("%-9.9s",t);
@@ -1507,11 +1549,11 @@ check_config()
     }
     if (intf_of(&Digipath[0]) == NULL) {
       fprintf(stderr,"ERROR: %s: first callsign in omni route is not one of my callsigns.\n",
-	      ax2asc(&Path[n].fsa_digipeater[0]));
+	      ax25_ntoa_pretty(&Path[n].fsa_digipeater[0]));
       exit(1);
     }
     for (j = 0; j < DIGISIZE; j++) {
-      printf("%-9.9s ",ax2asc(&Digipath[j]));
+      printf("%-9.9s ",ax25_ntoa_pretty(&Digipath[j]));
     }
     printf("\n");
   }
@@ -1520,12 +1562,12 @@ check_config()
     for (n = 0; n < 4; n++) {
       if (intf_of(&Path[n].fsa_digipeater[0]) == NULL) {
 	fprintf(stderr,"ERROR: %s: first callsign in %c route is not one of my callsigns.\n",
-		ax2asc(&Path[n].fsa_digipeater[0]),Dirs[n]);
+		ax25_ntoa_pretty(&Path[n].fsa_digipeater[0]),Dirs[n]);
 	exit(1);
       }
       printf("\n%c:        ",Dirs[n]);
       for (j = 0; j < Path[n].fsa_ax25.sax25_ndigis; j++)
-	printf("%-9.9s ",ax2asc(&Path[n].fsa_digipeater[j]));
+	printf("%-9.9s ",ax25_ntoa_pretty(&Path[n].fsa_digipeater[j]));
     }
     printf("\n");
   } else {
