@@ -22,7 +22,10 @@ struct ax_calls {
  *  on valid frame, 1 on valid frame that still has a non-repeated digi.
  */
 typedef enum {PK_INVALID=-1,PK_VALID=0,PK_VALDIGI=1} pk_val;
-extern pk_val parse_kiss(unsigned char **, int *, struct ax_calls *);
-extern int gen_kiss(unsigned char **, int *, struct ax_calls *);
+extern pk_val parse_raw_ax25(unsigned char **, int *, struct ax_calls *);
+extern int gen_raw_ax25(unsigned char **, int *, struct ax_calls *);
+
+/* pretty version of stand libax25's ax25_ntoa */
+extern char *ax25_ntoa_pretty(const ax25_address *a);
 
 #endif
